/*  defns.h --  structures, types, and macros for TopoVista  */

#ifndef DEFNS_H
#define DEFNS_H
/*
 * ============= Testing =================
 */
#define TESTING 1

/*
 * ============= Display =================
 */
#define HEIGHT_3D  400 /*618*/	/* height of 3-D (perspective) window */
#define WIDTH_3D   600 /*1000*/	/* width */
#define HEIGHT_2D  256		/* height of 2-D (overhead) window */
				/* (width is calculated) */
#define EYE_HEIGHT 2.0		/* eye height above terrain (meters) */
#define REF_HEIGHT 0.0		/* reference height above terrain (meters) */

/* terrain display modes (these must be -2 and -1) */
#define T_CONTOURS -1	/* color-coded elevation contours */
#define T_GRAY -2	/* grayscale relief only */

#define SEA_LEVEL 1		/*  sea level: if -SEA_LEVEL <=
				    elevation <= SEA_LEVEL, color as
				    ocean  */

/* surface generation modes */
#define NONE2D	0	/* flag for no 2D mesh */
#define WIRE2D	1
#define FOV2D	2
#define WIRE3D	3
#define FILL3D	4

/* Types of field of view */
#define WEDGE    0
#define FOUR_GON 4
#define FIVE_GON 5

/* relief shading */
#define COLORATION 0.7		/* color importance (vs. shading) (0. to 1.) */
#define CONTRAST   4.0		/* contrast stretching factor for relief */


typedef struct {		/* rgb spec for coloring */
    float red,green,blue;
} Color;



/*
 * ============== Heirarchy ===============
 */
#define MAXERR USHRT_MAX	/* maximum scaled triangle error */

/* triangle info flag bits */
#define SMALLER1	  1	/* Is neighbor 1 smaller? */
#define SMALLER2	  2	/* Is neighbor 2 smaller? */
#define BIGGER3		  4	/* Is neighbor 3 larger? */
#define SURFACE		  8	/* on surface? */
#define ALLINFOV	 16	/* completely in field of view? */
#define PARTINFOV	 32	/* partially in field of view? */
#define ALLINROD	 64	/* completely in rectangle of definition? */
#define PARTINROD	128	/* partially in rectangle of definition? */

/* info field access macros */
#define BORDER(hp,t) ((hp)->info[t] & PARTINROD)
#define INROD(hp,t) ((hp)->info[t] & ALLINROD)
#define PUTONSURFACE(hp,t) \
    (hp)->info[t] = ((hp)->info[t]&(ALLINROD|PARTINROD))|SURFACE
#define ONSURFACE(hp,t) ((hp)->info[t] & SURFACE)
#define INFOV(hp,t) ((hp)->info[t] & (ALLINFOV | PARTINFOV))
#define SMALLN1(hp,t) ((hp)->info[t] & SMALLER1)
#define SMALLN2(hp,t) ((hp)->info[t] & SMALLER2)
#define BIGN3(hp,t) ((hp)->info[t] & BIGGER3)

#define ISLEAF(hp,t) ((t)<<1 >= (hp)->size-1)

typedef struct {		/* triangle hierarchy */
  int depth;			/* maximum depth */
  int size;			/* number of triangles in hierarchy */
  int numSurfTri;		/* number of triangles on surface */
  unsigned short *err;	/* error of triangles */
  unsigned char *info;	/* surface,neighbor,infov of triangles */
} Hier;

typedef int TriIndex;

#define LCHILD(i) (2*(i)+1)	/* index of left child, given the parent */
#define RCHILD(i) (2*(i)+2)	/* index of right child, given the parent */
#define PARENT(i) (((i)-1)/2)	/* index of parent, given the child */

/* initial values */
#define IERRTHRESH	0.001
#define IERRTHRESHINC	0.00005
#define IDISTTHRESH	700.0
#define IDISTTHRESHINC	50.0
#define IVEXAGGER	1.0
#define IVEXAGGERINC	0.1

/* Lineal paths */
typedef struct Path{
  int n, allocation;		/* number of points and allocated size */
  Color color;			/* color of path */
  struct Path* next;		/* next path in list */
  Coord2f *p;			/* coordinates suitable for display */
} Path;
#endif	/* DEFNS_H */
