/*  globals.h -- global variables of TopoVista  */


/*
 *  variables prefaced by "global" are both declared and defined by this
 *  file, depending on how that macro is defined.
 */

#ifndef global
#define global extern
#define initial(v) /**/
#endif



/* windows (GL/GLUT handles) */

global int ctlWin;		/* control window */
global int ovhWin;		/* overhead ("2D") window */
global int pspWin;		/* perspective ("3D") window */


/* terrain data */

global Rgb **clayer;		/* array of color layers */
global char **layerName;	/* names of layers */
global int nLayers;		/* number of color layers */

global char *gemname;		/* name of elevation model */
global Em *gem;			/* global elevation model */

global int gemXmax, gemYmax;	/* maximum x, y indices in grid data */

global double xmeters, ymeters;	/* dimensions of data in meters */

global double m2grid;		/* scales vertical meters to grid units */

global double dz2contour;      	/* scales data to contour level */

global double high_water,low_water;/* SEA_LEVEL,-SEA_LEVEL in data units */

global double err2grid;		/* scales 0..MAXERR to grid units */

global Hier *hierarchy;		/* hierarchy pointer */

global Path *paths;		/* pointer to linked list of paths */

/*  viewing options  */

global int colorLayer;		/* color layer to display */

global int twoDMesh	initial(NONE2D); /* show 2-D mesh? */
global int threeDMesh	initial(0);	/* show 3-D mesh? */

global int eyeUp	initial(1);	/* is eye above data plane? */

/* vertical exaggeration */
global double vexagger		initial(IVEXAGGER);
global double vexaggerInc	initial(IVEXAGGERINC);

global int cullBackface	initial(0);	/* cull backfaces */
global int cullFOV	initial(0);	/* cull outside of FOV */
global int smoothSurface initial(1);	/* initially show smooth surface */

/* parameters used to judge triangle fitness */
/* Triangle err/distance from eye must be < errThresh */
global double errThresh          initial(IERRTHRESH);
global double errThreshInc       initial(IERRTHRESHINC);

/* Triangles < k*distThresh meters from eye must have size < 2^k */
global double distThresh	initial(IDISTTHRESH);
global double distThreshInc	initial(IDISTTHRESHINC);


/* current viewpoint */

global Coord3f eye;	/* eye location */
global Coord3f ref;	/* reference point */

global int fovType;	/* shape of FOV (see defns.h) */
global Coord2f fovLLpt, fovULpt, fovLRpt, fovURpt;
global Coord2f fovTnorm, fovBLnorm, fovBRnorm, fovLnorm, fovRnorm;


/* other */

global int numDisplayTri;		/* number of displayed triangles */

global Coord2d se, sw, ne, nw;

global int tflag	initial(0);	/* calculate timing info? */

#ifdef TESTING
global int doingLoop	initial(0);	/* doing a loop around perimeter? */
global int doRedraw	initial(1);	/* redraw 3D & 2D view during loop? */
global int triPerLoop	initial(0);	/* total #tri drawn during loop */
#endif


/* defined in main.c (too complex for macros above): */

extern Color elevMap[];		/* elevation colors */
extern int elevCount;		/* number of elevation colors */


/* these macros should not be used elsewhere */
#undef global
#undef initial
