/*  neighbor.c -- find neighbors of triangle  */

#include "topo.h"

/* The following chart describes the neighbor calculations in a
   recursive fashion: (Note: null concatenated with anything is null)

   N1(null) = N1(0) = N1(1) = null
   N1(w0) = N3(w)1
   N1(w1) = w0

   N2(null) = N2(0) = N2(1) = null
   N2(w0) = w1
   N2(w1) = N3(w)0

   N3(null) = null
   N3(0) = 1
   N3(1) = 0
   N3(w0) = N2(w)1
   N3(w1) = N1(w)0
*/

int neighbor[3][3] = {{0, 0, 0}, {0, 0, 0}, {0, 2, 1}};

#define SIEVE0 0xAAAAAAAA
#define SIEVE1 0x55555555

#define CUZZ(p)				\
  c = ((p << 1) ^ p) & SIEVE0;		\
  a = c | (c << 1) | 1;			\
  b = (a + 1) ^ a;			\
  if (b < (p>>1)) {			\
    n = p ^ b;				\
  } else if (((p & SIEVE0) << 1) > p) {	\
    if (((p & SIEVE1) << 2) > p) {	\
      n = p ^ (b>>1);			\
    } else {				\
      n = p ^ (b>>3);			\
    }					\
  } else {				\
    return 0;				\
  }					 

TriIndex nbr1(Hier *hp, TriIndex t)
     /* Returns index of neighbor opposite vertex 1 of t */
{
  unsigned int a,b,c,p;
  TriIndex n;

  if (t <= 2) return neighbor[0][t];
  p = ((t+1)<<1)|1;
  CUZZ(p);
  n = (n >> 1)-1;
  if (n && SMALLN1(hp, t)) {
    return LCHILD(n);
  } else {
    return n;
  }
}

TriIndex nbr2(Hier *hp, TriIndex t)
     /* Returns index of neighbor opposite vertex 2 of t */
{
  unsigned int a,b,c,p;
  TriIndex n;
  
  if (t <= 2) return neighbor[1][t];
  p = (t+1)<<1;
  CUZZ(p);
  n = (n>>1)-1;
  if (n && SMALLN2(hp, t)) {
    return RCHILD(n);
  } else {
    return n;
  }
}

TriIndex nbr3(Hier *hp, TriIndex t)
     /* Returns index of neighbor opposite vertex 3 of t */
{
  unsigned int a,b,c,p;
  TriIndex n;

  if (t <= 2) return neighbor[2][t];
  p = t+1;
  CUZZ(p);
  --n;
  if (n && BIGN3(hp, t)) {
    return PARENT(n);
  } else {
    return n;
  }
}
