/*  normal.c -- get the normal of surface  */

#ifdef WIN32
#include <windows.h>
#endif
#include "topo.h"
#include <GL/gl.h>

static void DFSNormal(TriIndex t, Coord2d p1, Coord2d p2, Coord2d p3);
static void triNormal(TriIndex t, Coord2d p1, Coord2d p2, Coord2d p3);



static double zscale;
static Hier *hp;

#define GET_NORMAL {				\
  nx = (y2-y1)*(sz3-sz1) - (sz2-sz1)*(y3-y1);	\
  ny = (sz2-sz1)*(x3-x1) - (x2-x1)*(sz3-sz1);	\
  nz = (float)((x2-x1)*(y3-y1) - (y2-y1)*(x3-x1));	\
}

/*Modified by Xiangyu Zhang , for Normal*/
void surfaceNormal(Hier *h)
{
  int length;
  hp = h;
  zscale = vexagger * m2grid * gem->dz2m;
  numDisplayTri = 0;
  length = gem->width * gem->height;
  memset((void*)gem->normal,0,3*length*sizeof(float));
  DFSNormal(LCHILD(0), sw, ne, nw);	/* NW corner of square */
  DFSNormal(RCHILD(0), ne, sw, se);	/* SE corner of square */
}



static void DFSNormal(TriIndex t, Coord2d p1, Coord2d p2, Coord2d p3)
{
  Coord2d mid;

  if (ONSURFACE(hp,t)) {
    triNormal(t, p1, p2, p3);
  } else {
    assert(!ISLEAF(hp, t));
		
    MIDPOINT(p1, p2, mid);
    DFSNormal(LCHILD(t), p3, p1, mid);
    DFSNormal(RCHILD(t), p2, p3, mid);
  }
}

static void triNormal(TriIndex t, Coord2d p1, Coord2d p2, Coord2d p3)
{
  int x1,x2,x3,y1,y2,y3,z1,z2,z3;
  float sz1, sz2, sz3;
  float nx, ny, nz;

  if (INROD(hp,t) && (!cullFOV || INFOV(hp,t))) {
			
    x1=p1[0];
    y1=p1[1];
    z1=EMDATA(gem, x1, y1);
    sz1 = (float)zscale*z1;

    x2=p2[0];
    y2=p2[1];
    z2=EMDATA(gem, x2, y2);
    sz2 = (float)zscale*z2;

    x3=p3[0];
    y3=p3[1];
    z3=EMDATA(gem, x3, y3);
    sz3 = (float)zscale*z3;

    GET_NORMAL;
    EMNORMAL(gem,x1,y1,0)+=nx;
    EMNORMAL(gem,x1,y1,1)+=ny;
    EMNORMAL(gem,x1,y1,2)+=nz;

    EMNORMAL(gem,x2,y2,0)+=nx;
    EMNORMAL(gem,x2,y2,1)+=ny;
    EMNORMAL(gem,x2,y2,2)+=nz;

    EMNORMAL(gem,x3,y3,0)+=nx;
    EMNORMAL(gem,x3,y3,1)+=ny;
    EMNORMAL(gem,x3,y3,2)+=nz;

   }

}
