/*  print.c -- print routines */

#include "topo.h"

static Hier *hp;	/* global so not in each activation record*/
static double zscale;
static FILE *fp;

static void printTri(Coord2d p1, Coord2d p2, Coord2d p3)
{
  double x1,x2,x3,y1,y2,y3,z1,z2,z3;
  double px, py, pz, len;
  double nx, ny, nz;
			
  x1=p1[0];
  y1=p1[1];
  z1=zscale*EMDATA(gem, x1, y1);

  x2=p2[0];
  y2=p2[1];
  z2=zscale*EMDATA(gem, x2, y2);

  x3=p3[0];
  y3=p3[1];
  z3=zscale*EMDATA(gem, x3, y3);

  px = (y2-y1)*(z3-z1) - (z2-z1)*(y3-y1);
  py = (z2-z1)*(x3-x1) - (x2-x1)*(z3-z1);
  pz = (x2-x1)*(y3-y1) - (y2-y1)*(x3-x1);
  len = px * px + py * py + pz * pz;
  len = sqrt(len);
  nx = px / len;
  ny = py / len;
  nz = pz / len;

  fprintf(fp, " facet normal %f %f %f\n", nx, ny, nz);
  fprintf(fp, "  outer loop\n");
  fprintf(fp, "   vertex %f %f %f\n", x1, y1, z1);
  fprintf(fp, "   vertex %f %f %f\n", x2, y2, z2);
  fprintf(fp, "   vertex %f %f %f\n", x3, y3, z3);
  fprintf(fp, "  endloop\n");
  fprintf(fp, " endfacet\n");
}

static void printDFS(TriIndex t, Coord2d p1, Coord2d p2, Coord2d p3)
{
  Coord2d mid;

  if( ONSURFACE(hp, t) ) {
    if( INROD(hp,t) && (!cullFOV || INFOV(hp,t)) ) {
      printTri(p1, p2, p3);
    }
  } else if( !ISLEAF(hp,t) ) {
    MIDPOINT(p1, p2, mid);
    printDFS(LCHILD(t), p3, p1, mid);
    printDFS(RCHILD(t), p2, p3, mid);
  }
}

void printSurface(Hier *h, char *fname)
{
  hp = h;
  zscale = vexagger * m2grid * gem->dz2m;
  fp = fopen(fname,"w");
  if( fp == NULL ) {
    fprintf(stderr, "Can't open %s for writing.\n", fname);
    return;
  }
  fprintf(stderr, "Writing triangle list to %s...", fname);

  fprintf(fp, "solid %s created by Topovista\n", fname);
  printDFS(LCHILD(0), sw, ne, nw);
  printDFS(RCHILD(0), ne, sw, se);
  fprintf(fp, "endsolid %s\n", fname);

  fprintf(stderr, "Done\n");

  fclose(fp);
}
