/*  surface.c -- compute surface based on error criteria  */


#include "topo.h"

static double etGrid,dtGrid;
static Hier *hp;
static int depth;


/*static int tooCoarse(TriIndex t, Coord2d p1, Coord2d p2, Coord2d p3)*/
static int tooCoarse(TriIndex t, Coord2d p3)
{
  int h,dx,dy,d;

  if (!INFOV(hp,t))		/* if not in field of view */
    return 0;


  /* Check distance from eye */
  dx = (int)(p3[0] - eye[0]);		/* dx and dy in grid units */
  dy = (int)(p3[1] - eye[1]);
  d = dx * dx + dy * dy;
  if (d == 0)				/* if right below eye */
    return 1;

  d = (int)sqrt((double)d);		/* d in grid units */
  if (d  < dtGrid * (hp->depth - depth))
    return 1;


  /* Check error threshold */
  h = hp->err[t];

  if (h > d * etGrid)
    return 1;

  return 0;
}

static void rodSplit(TriIndex t, Coord2d p1, Coord2d p2, Coord2d p3)
     /* split triangles if they cross Rectangle Of Definition *or* if
	they are too coarse. */
{
  Coord2d mid;

  ++depth;
  if (!ISLEAF(hp, t) && (BORDER(hp,t) || tooCoarse(t, p3))) {
    if (ONSURFACE(hp,t)) {
      split(hp,t);
    }
    MIDPOINT(p1, p2, mid);
    rodSplit(LCHILD(t), p3, p1, mid);
    rodSplit(RCHILD(t), p2, p3, mid);
  }
  --depth;
}


static void eyeSplit(TriIndex t, Coord2d p1, Coord2d p2, Coord2d p3)
     /* splits triangle if it contains the eye position */
{
  Coord2d mid;

  ++depth;
  if (!ISLEAF(hp, t)) {
    if (ONSURFACE(hp,t)) {
      split(hp, t);
    }
    MIDPOINT(p1, p2, mid);
    if (LEFT_TURN(mid, p3, eye)) {
      eyeSplit(LCHILD(t), p3, p1, mid);
      rodSplit(RCHILD(t), p2, p3, mid);
    } else {
      eyeSplit(RCHILD(t), p2, p3, mid);
      rodSplit(LCHILD(t), p3, p1, mid);
    }
  }
  --depth;
}

void errSurface(Hier *h)
{
  double starttime=0.0,endtime;
  hp = h;

  if (tflag) {
    starttime=getTime();
  }

  etGrid = errThresh / err2grid;	/* set error parameters */
  dtGrid = distThresh * m2grid;

  /* SURFACE starts as triangles LCHILD(0) and RCHILD(0) */
  PUTONSURFACE(hp, LCHILD(0));
  PUTONSURFACE(hp, RCHILD(0));
  hp->numSurfTri = 2;
  hp->info[LCHILD(0)] |= triInFOV(LCHILD(0));
  hp->info[RCHILD(0)] |= triInFOV(RCHILD(0));

  depth = 1;
  if (LEFT_TURN(sw, ne, eye)) {
    eyeSplit(LCHILD(0), sw, ne, nw);	/* NW half of square */
    rodSplit(RCHILD(0), ne, sw, se);	/* SE half of square */
  } else {
    eyeSplit(RCHILD(0), ne, sw, se);	/* SE half of square */
    rodSplit(LCHILD(0), sw, ne, nw);	/* NW half of square */
  }

  if (tflag) {
    endtime=getTime();
    fprintf(stderr,"Surface built in %f seconds.\n",endtime-starttime);
  }
}
