#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include "basic.h"

void nospace(void)
{
  fprintf(stderr,"Out of Memory.\n");
  exit(1);
}

/* Under Windows this gets the clock time using _ftime(). */
/* Under UNIX this gets the resource usage time using getrusage(). */
double getTime(void)
{
#ifdef WIN32
  struct _timeb tstruct;

  _ftime( &tstruct );
  return( (double)tstruct.time + (double)tstruct.millitm/1000.);
#else
  struct rusage r;

  getrusage(RUSAGE_SELF,&r);
  return((double)r.ru_utime.tv_sec + (double)r.ru_utime.tv_usec/1000000.);
#endif
}

#ifndef NO_ZLIB
#  define MAX_INPUT_BUFFER 1
#  define MAX_FILENAME_LEN 1024
#  ifndef GZ_SUFFIX
#    define GZ_SUFFIX ".gz"
#  endif
#  define SUFFIX_LEN (sizeof(GZ_SUFFIX)-1)

static int buf[MAX_INPUT_BUFFER];
static int bufsize = 0;

int mygetc( gzFile g )
{
  if( bufsize > 0 ) {
    return buf[--bufsize];
  }
  return gzgetc(g);
}


int myungetc( int c, gzFile g )
{
  if( bufsize >= MAX_INPUT_BUFFER ) return EOF;
  buf[bufsize++] = c;
  return c;
}

FILETYPE myfopen( char *path, char *mode )
{
  char fn[MAX_FILENAME_LEN];
  int len = strlen(path);
  FILETYPE g;

  g = gzopen(path, mode);
  if( g ) return g;
  strncpy(fn, path, MAX_FILENAME_LEN-SUFFIX_LEN);
  if( len < SUFFIX_LEN || strcmp(path+len-SUFFIX_LEN, GZ_SUFFIX) ) {
    strcat(fn, GZ_SUFFIX);
  }
  return gzopen(fn, mode);
}
#endif

FILETYPE open4read( char *path )
{
  if( path == NULL ) {
    SET_BINARY_MODE(stdin);
    return FDOPEN(fileno(stdin), "rb");
  } else {
    return FOPEN(path, "rb");
  }
}
